#ifndef __LDMINFO_H_
#define __LDMINFO_H_

#include <linux/version.h>
#include <linux/kdev_t.h>
#include <linux/blkdev.h>
#include <linux/types.h>
#include <linux/fs.h>
#include <linux/genhd.h>
#include <linux/list.h>
#include <features.h>

#define __ssize_t_defined
#include <stdio.h>
#include <unistd.h>

#include "../linux/fs/partitions/ldm.h"

#ifndef minor
#define minor MINOR
#endif

#ifndef major
#define major MAJOR
#endif

#ifndef mk_kdev
#define mk_kdev MKDEV
#endif

#ifndef BOOL
typedef unsigned char	BOOL;
#endif

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,3)
#define kdev_val(X) (unsigned short)(X)
#else
#define kdev_val(X) ((X).value)
#endif

#define LDM_CRIT	KERN_CRIT
#define LDM_ERR		KERN_ERR
#define LDM_INFO	KERN_INFO
#define LDM_DEBUG	KERN_DEBUG

extern int * hardsect_size[255];
extern int * blk_size     [255];
extern int * blksize_size [255];

extern struct hd_struct	hd      [256];
extern int              hd_sizes[256];
extern struct gendisk   hd_gendisk;

extern int device;
extern int debug;

extern int ldm_a;
extern int ldm_f;

extern struct proc_dir_entry *proc_root_fs;

struct tm
{
	int tm_sec;		/* Seconds */
	int tm_min;		/* Minutes */
	int tm_hour;		/* Hours */
	int tm_mday;		/* Day */
	int tm_mon;		/* Month */
	int tm_year;		/* Year */
	int tm_wday;		/* Day of week */
	int tm_yday;		/* Days in year */
	int tm_isdst;		/* DST */
	long int tm_gmtoff;	/* Seconds east of UTC */
	const char *tm_zone;	/* Timezone abbreviation */
};

void dump_database (char *name, struct ldmdb *ldb);
void copy_database (char *file, int fd, long long size);
struct buffer_head * ldm_bread (kdev_t dev, int block, int size);

BOOL ldm_create_partition (struct gendisk *hd, int minor, int start, int size);
BOOL ldm_create_data_partitions (struct gendisk *hd, unsigned long first_sector, int first_minor, const struct ldmdb *ldb, const struct vblk *disk);
int ldm_validate_privheads (struct block_device *bdev, unsigned long base, const struct privhead *ph1, struct gendisk *hd, unsigned long first_minor);
int ldm_validate_tocblocks (struct block_device *bdev, unsigned long base, struct ldmdb *ldb);
int ldm_validate_vmdb (struct block_device *bdev, unsigned long base, struct ldmdb *ldb);
int ldm_get_buffers (struct block_device *bdev, unsigned long base, struct vmdb *vm, list_t *lh_bh, list_t *lh_vl);
struct vblk * ldm_get_disk_objid (const struct ldmdb *ldb);
void ldm_free_buffers (list_t *bh);
void ldm_free_vblks (list_t *vb);
int ldm_get_vblks(struct block_device *bdev, unsigned long base, struct ldmdb *ldb);
int ldm_combine_vblks (struct ldmdb *ldb);
int ldm_validate_partition_table (struct block_device *bdev);

void		exit	(int);
int		open64	(const char *file, int oflag, ...);
long long	lseek64 (int fd, long long offset, int whence);
ssize_t		read	(int fd, void *buf, size_t count);
int		close	(int fd);
struct tm *	localtime(const time_t *timer);
int		isdigit	(int c);
char *		basename(const char *filename);

#endif // __LDMINFO_H_

